// file      : xsd/cxx/post.hxx
// copyright : Copyright (c) 2005-2014 Code Synthesis Tools CC
// license   : GNU GPL v2 + exceptions; see accompanying LICENSE file

#ifdef _MSC_VER
#  if (_MSC_VER >= 1400)
#    include <xsd/cxx/compilers/vc-8/post.hxx>
#  endif
#endif
