// file      : xsd/cxx/xml/exceptions.hxx
// copyright : Copyright (c) 2005-2014 Code Synthesis Tools CC
// license   : GNU GPL v2 + exceptions; see accompanying LICENSE file

#ifndef XSD_CXX_XML_EXCEPTIONS_HXX
#define XSD_CXX_XML_EXCEPTIONS_HXX

namespace xsd
{
  namespace cxx
  {
    namespace xml
    {
      struct invalid_utf16_string {};
    }
  }
}

#endif // XSD_CXX_XML_EXCEPTIONS_HXX
