// file      : xsd/cxx/tree/istream-fwd.hxx
// copyright : Copyright (c) 2005-2014 Code Synthesis Tools CC
// license   : GNU GPL v2 + exceptions; see accompanying LICENSE file

#ifndef XSD_CXX_TREE_ISTREAM_FWD_HXX
#define XSD_CXX_TREE_ISTREAM_FWD_HXX

namespace xsd
{
  namespace cxx
  {
    namespace tree
    {
      template<typename S>
      class istream;
    }
  }
}

#endif  // XSD_CXX_TREE_ISTREAM_FWD_HXX
