// file      : xsd/cxx/tree/parsing.hxx
// copyright : Copyright (c) 2005-2014 Code Synthesis Tools CC
// license   : GNU GPL v2 + exceptions; see accompanying LICENSE file

#ifndef XSD_CXX_TREE_PARSING_HXX
#define XSD_CXX_TREE_PARSING_HXX

#include <xsd/cxx/tree/parsing.txx>
#include <xsd/cxx/tree/parsing/date-time.txx>

#endif  // XSD_CXX_TREE_PARSING_HXX
